/*! \file abbreviate.c

    \brief remove "uninteresting" lines from a data file so that it can fit within an Excel plot.

*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

int main( int argc , char ** argv )
{
    FILE * fpIn = fopen( "signal.csv" , "r" ) ;
    assert( fpIn != 0 ) ;

    FILE * fpOut = fopen( "signal-abbr.csv" , "w" ) ;
    assert( fpOut != 0 ) ;

    static const int numChars = 1024 ;
    char strLines[ 2 ][ numChars ] ;

    /* Prime the routine: Read a line of text into storage buffer.
       Note the index of 1 here.  Index 0 is the "current" line.
       Index 1 is the "previous" line.
       In the loop, the first line of the file will be considered as
       the first "previous" line.
    */
    fgets( strLines[ 1 ] , numChars , fpIn ) ;

    int iLine                   = 0 ;
    int bPrevLineWasInteresting = 1 ;
    int bPrevLineWasOutput      = 1 ;

    for( iLine = 0 ; ! feof( fpIn ) ; ++ iLine )
    {
        /* Read in a new line */
        fgets( strLines[ 0 ] , numChars , fpIn ) ;

        /* Parse the current line into fields */
        char fields[ 14 ][ 128 ] ;
        sscanf( strLines[ 0 ] , "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],\n" ,
          fields[  0 ] , fields[  1 ] , fields[  2 ] , fields[  3 ] , fields[  4 ] ,
          fields[  5 ] , fields[  6 ] , fields[  7 ] , fields[  8 ] , fields[  9 ] ,
          fields[ 10 ] , fields[ 11 ] , fields[ 12 ] , fields[ 13 ] ) ;

        /* Extract info from certain fields */
        const int   iChannel = atoi( fields[  2 ] ) ;
        const int   iFreq1   = atoi( fields[  3 ] ) ;
        const int   iRate1   = atoi( fields[  5 ] ) ;
        const float fSig1    = atof( fields[  6 ] ) ;
        const float fSnr     = atof( fields[  7 ] ) ;
        const int   iFreq2   = atoi( fields[ 10 ] ) ;
        const int   iRate2   = atoi( fields[ 12 ] ) ;
        const float fSig2    = atof( fields[ 13 ] ) ;

        char status[ 2 ][ 128 ] ;
        sscanf( fields[ 1 ] , " %s " , status[ 0 ] ) ;
        sscanf( fields[ 8 ] , " %s " , status[ 1 ] ) ;

        /* Signal Level Scale according to ispgeek posting to dslreports.com:

           Downstream power
           15 and higher (BAD)
           10 to 14 (Borderline)
           9 to -9 (Good)
           -10 to -14 (Borderline)
           -15 and lower (BAD)

           Upstream power
           55 and higher (BAD)
           50 to 54 (Borderline)
           36 to 49 (GOOD)
           35 to 31 (Borderline)
           30 and lower (BAD)
        */

        /* Determine whether the current line is interesting */
        const int bInteresting =
                ( iChannel < 121     )
            ||  ( iChannel > 124     )

            ||  ( iFreq1 < 609000000 )
            ||  ( iFreq1 > 627000000 )
            ||  ( iRate1 < 10256384  )
            ||  ( iRate1 > 10343584  )
            ||  ( fSig1 < -13.0f )
            ||  ( fSig1 > 6.0f )

            ||  ( iFreq2 < 20600000  )
            ||  ( iFreq2 > 33008000  )
            ||  ( iRate2 < 1024000   )
            ||  ( iRate2 > 1024000   )
            ||  ( fSig2 < 33.0f )
            ||  ( fSig2 > 51.0f )

            ||  ( fSnr < 20.0f ) 
            ||  ( fSnr > 40.0f ) 

            ||  ( 0 == iLine )
            ||  ( strcmp( status[0] , "Operational" ) )
            ||  ( strcmp( status[1] , "Operational" ) )
            ;

#if 0
        if( bInteresting )
        {
            printf("line %i " , iLine ) ;
            printf(" was interesting: " ) ;
            printf(" status1='%s' status2='%s' chan=%i f1=%i r1=%i s1=%g f2=%i r2=%i s2=%g snr=%g\n"
              , fields[ 1 ] , fields[ 8 ] , iChannel , iFreq1 , iRate1 , fSig1 , iFreq2 , iRate2 , fSig2 , fSnr ) ;
        #if 0
            int iField ;
            for( iField = 0 ; iField < 14 ; ++ iField )
            {
              printf("field[%2i]='%s'\n" , iField , fields[iField]) ;
            }
        #endif
        }
#endif

        if( ( ( bInteresting || bPrevLineWasInteresting ) ) && ( ! bPrevLineWasOutput ) )
        {   /* Either current or previous line was interesting. */
            /* Print previous line. */
            fprintf( fpOut , "%s" , strLines[ 1 ] ) ;
        }

        if( bPrevLineWasInteresting && ( ! bInteresting ) )
        {   /* Previous line was interesting but current line is not. */
            /* Print current line. */
            fprintf( fpOut , "%s" , strLines[ 0 ] ) ;
            bPrevLineWasOutput = 1 ;
        }
        else
        {
            bPrevLineWasOutput = 0 ;
        }

        /* Remember whether this line was interesting */
        bPrevLineWasInteresting = bInteresting ;

        /* Copy current line to storage buffer for next iteration. */
        strcpy( strLines[ 1 ] , strLines[ 0 ] ) ;
    }

    /* Always print out last line */
    fprintf( fpOut , "%s" , strLines[ 1 ] ) ;

    fclose( fpIn ) ;
    return 0 ;
}

